"use strict";
// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
Object.defineProperty(exports, "__esModule", { value: true });
exports.App = exports.Lambdas = void 0;
class Lambdas {
    static pipe(handler1, handler2) {
        return (event, context, callback) => {
            handler1(event, context, (error, evt2) => {
                if (error) {
                    callback(error, null);
                }
                else {
                    handler2(evt2, context, (err, res) => callback(err, res));
                }
            });
        };
    }
}
exports.Lambdas = Lambdas;
class App {
    static cli(options, handler) {
        if (App.suppressRun) {
            return this;
        }
        const optsMap = new Map();
        for (let op of options) {
            let matched = false;
            let isOptional = false;
            if (op.startsWith('?')) {
                op = op.substring(1);
                isOptional = true;
            }
            for (const arg of process.argv.slice(2)) {
                if (op.endsWith('*')) {
                    if (arg.startsWith(op.substring(0, op.length - 1))) {
                        matched = true;
                        optsMap.set(op, arg.substring(op.length - 1));
                        break;
                    }
                }
                else {
                    if (arg === op) {
                        matched = true;
                        optsMap.set(op, arg.substring(op.length));
                        break;
                    }
                }
            }
            if (!isOptional && !matched) {
                return this;
            }
        }
        handler(optsMap);
        App.suppressRun = true;
        return this;
    }
    static route(name, lambda) {
        App.lambdas.set(name, lambda);
        return this;
    }
    static run(context, lambda) {
        if (App.suppressRun) {
            return;
        }
        process.stdin.resume();
        process.stdin.setEncoding('utf8');
        let inputData = '';
        process.stdin.on('data', (chunk) => {
            inputData += chunk;
        });
        process.stdin.on('end', () => {
            const write = (data, exitCode) => {
                const done = process.stdout.write(data);
                if (done) {
                    process.exit(exitCode);
                }
                else {
                    process.stdout.once('drain', () => {
                        process.exit(exitCode);
                    });
                }
            };
            try {
                const event = JSON.parse(inputData);
                // tslint:disable-next-line:variable-name
                let lambda_ = lambda;
                if (!lambda_) {
                    if (event && event.eventName) {
                        lambda_ = App.lambdas.get(event.eventName);
                    }
                }
                lambda_ = lambda_ || ((evt, ctx, cb) => {
                    cb({ message: `Error: event name ${(event && event.eventName) || ''} is not found.` }, null);
                });
                lambda_(event, context, (error, result) => {
                    if (error) {
                        write(String(error), -1);
                    }
                    else {
                        write(result, 0);
                    }
                });
            }
            catch (e) {
                write(String(e), -1);
            }
        });
    }
}
exports.App = App;
App.lambdas = new Map();
App.suppressRun = false;
//# sourceMappingURL=app.js.map