import * as RedAgate from './red-agate';
import { ShapeProps, Shape } from './tags/Shape';
export interface AssetProps extends RedAgate.ComponentProps {
    src: string;
    contextName: string;
}
export declare class Asset extends RedAgate.RedAgatePhantomComponent<AssetProps> {
    constructor(props: AssetProps);
    private content;
    defer(): Promise<void>;
    beforeRender(contexts: Map<string, any>): void;
    render(contexts: Map<string, any>, children: string): string;
}
export interface ImageInstantProps extends ShapeProps {
    use?: undefined;
    srcContext?: undefined;
    noDownload?: true;
    src: string;
    unit?: string;
    width: number;
    height: number;
    asAsset?: false;
    alt?: string;
    ismap?: true;
    longdesc?: string;
    sizes?: string;
    usemap?: string;
}
export interface ImageInstantFromCtxProps extends ShapeProps {
    use?: undefined;
    srcContext: string;
    noDownload?: true;
    src?: undefined;
    unit?: string;
    width: number;
    height: number;
    asAsset?: false;
    alt?: string;
    ismap?: true;
    longdesc?: string;
    sizes?: string;
    usemap?: string;
}
export interface ImageAssetProps extends ShapeProps {
    id: string;
    use?: undefined;
    srcContext?: undefined;
    noDownload?: undefined;
    src: string;
    unit?: string;
    width?: undefined;
    height?: undefined;
    asAsset: true;
    alt?: string;
    ismap?: true;
    longdesc?: string;
    sizes?: string;
    usemap?: string;
}
export interface ImageAssetFromCtxProps extends ShapeProps {
    id: string;
    use?: undefined;
    srcContext: string;
    noDownload?: undefined;
    src?: undefined;
    unit?: string;
    width?: undefined;
    height?: undefined;
    asAsset: true;
    alt?: string;
    ismap?: true;
    longdesc?: string;
    sizes?: string;
    usemap?: string;
}
export interface ImageRefProps extends ShapeProps {
    use: string;
    srcContext?: undefined;
    noDownload?: undefined;
    src?: undefined;
    unit?: string;
    width: number;
    height: number;
    asAsset?: false;
    alt?: string;
    ismap?: true;
    longdesc?: string;
    sizes?: string;
    usemap?: string;
}
export declare type ImageProps = ImageInstantProps | ImageInstantFromCtxProps | ImageAssetProps | ImageAssetFromCtxProps | ImageRefProps;
export declare class Image extends Shape<ImageProps> {
    constructor(props: ImageProps);
    transform(): {};
    private content;
    defer(): Promise<void>;
    private renderAsHtml;
    private renderAsSvg;
    render(contexts: Map<string, any>, children: string): string;
}
export interface ScriptProps extends RedAgate.ComponentProps {
    noDownload?: true;
    src: string;
    text?: string;
    type?: string;
    async?: true;
    defer?: true;
    crossorigin?: 'anonymous' | 'use-credentials';
}
export declare class Script extends RedAgate.RedAgatePhantomComponent<ScriptProps> {
    constructor(props: ScriptProps);
    private content;
    defer(): Promise<void>;
    render(contexts: Map<string, any>, children: string): string;
}
export interface StyleProps extends RedAgate.ComponentProps {
    noDownload?: true;
    src: string;
    type?: string;
    media?: string;
    nonce?: string;
    title?: string;
}
export declare class Style extends RedAgate.RedAgatePhantomComponent<StyleProps> {
    constructor(props: StyleProps);
    private content;
    defer(): Promise<void>;
    render(contexts: Map<string, any>, children: string): string;
}
export declare type FontProps = StyleProps;
export declare const Font: typeof Style;
export interface SingleFontProps extends RedAgate.ComponentProps {
    noDownload?: true;
    fontFamily: string;
    localNames?: string[];
    fontStyle: "normal" | "italic" | "oblique";
    fontWeight: 100 | 200 | 300 | 400 | 500 | 600 | 700 | 800 | 900 | "normal" | "bold" | "lighter" | "bolder";
    src: string;
    unicodeRange: string;
    format: "woff" | "woff2" | "truetype" | "opentype" | "embedded-opentype" | "svg";
}
export declare class SingleFont extends RedAgate.RedAgatePhantomComponent<SingleFontProps> {
    constructor(props: SingleFontProps);
    private content;
    defer(): Promise<void>;
    render(contexts: Map<string, any>, children: string): string;
}
