"use strict";
// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
Object.defineProperty(exports, "__esModule", { value: true });
exports.SingleFont = exports.Font = exports.Style = exports.Script = exports.Image = exports.Asset = void 0;
const RedAgate = require("./red-agate");
const Shape_1 = require("./tags/Shape");
const Base64_1 = require("red-agate-util/modules/convert/Base64");
const TextEncoding_1 = require("red-agate-util/modules/convert/TextEncoding");
const FileFetcher_1 = require("red-agate-util/modules/io/FileFetcher");
const Logger_1 = require("red-agate-util/modules/io/Logger");
const Uri_1 = require("red-agate-util/modules/io/Uri");
class Asset extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.content = '';
    }
    defer() {
        if (this.props.src === void 0 || this.props.src === null || this.props.src === '') {
            return Promise.resolve();
        }
        else {
            const src = this.props.src;
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher_1.FileFetcher.fetchLocation(this.props.src)
                        .then((result) => {
                        let contentType = result.contentType;
                        if (contentType === null) {
                            const pathExt = Uri_1.Uri.getPathExt(src);
                            contentType = Uri_1.Uri.pathExtToContentType(pathExt);
                            if (contentType === null) {
                                Logger_1.Logger.log("Asset#defer:unknown content-type:" + pathExt);
                                reject("unknown content-type");
                                return;
                            }
                        }
                        this.content = `data:${contentType};base64,` + Base64_1.Base64.encode(result.data, 120);
                        resolve();
                    })
                        .catch((e) => {
                        Logger_1.Logger.log("Asset#defer:catch:" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger_1.Logger.log("Asset#defer:catch:" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    beforeRender(contexts) {
        this.setContext(contexts, Shape_1.CONTEXT_ASSET_ + this.props.contextName, this.content);
    }
    render(contexts, children) {
        return '';
    }
}
exports.Asset = Asset;
class Image extends Shape_1.Shape {
    constructor(props) {
        super(props);
        this.content = '';
    }
    transform() {
        return ((this.props.children !== null && this.props.children !== void 0) ?
            this.props.children : []);
    }
    defer() {
        if (this.props.use !== void 0 && this.props.use !== null && this.props.use !== "") {
            return Promise.resolve();
        }
        else if (this.content !== null && this.content !== void 0 && this.content !== "") {
            return Promise.resolve();
        }
        else if (this.props.src === null || this.props.src === void 0 || this.props.src === "") {
            return Promise.resolve();
        }
        else if (this.props.noDownload) {
            this.content = this.props.src;
            return Promise.resolve();
        }
        else {
            const src = this.props.src;
            const promise = new Promise((resolve, reject) => {
                FileFetcher_1.FileFetcher.fetchLocation(src)
                    .then((result) => {
                    let contentType = result.contentType;
                    if (contentType === null) {
                        const pathExt = Uri_1.Uri.getPathExt(src);
                        contentType = Uri_1.Uri.pathExtToContentType(pathExt);
                        if (contentType === null) {
                            Logger_1.Logger.log("Image#defer:unknown content-type:" + pathExt);
                            reject("unknown content-type");
                            return;
                        }
                    }
                    this.content = `data:${contentType};base64,` + Base64_1.Base64.encode(result.data, 120);
                    resolve();
                })
                    .catch((e) => {
                    Logger_1.Logger.log("Image#defer:" + e);
                    reject(e);
                });
            });
            return promise;
        }
    }
    renderAsHtml(contexts) {
        let r = '';
        if (this.props.asAsset) {
            r = `<script>(Function("return this")())['#img_asset_mb3vhWjBUxDX__${this.props.id}']="${this.content.replace(/\r/g, '').replace(/\n/g, '')}";</script>`;
        }
        else {
            if (this.props.use !== void 0 && this.props.use !== null && this.props.use !== "") {
                const id = this.props.id || `__img_node_id_mb3vhWjBUxDX_${this.props.__nodeId}`;
                r = `<img id="${id}" style="width:${this.props.width}${this.props.unit || 'px'};height:${this.props.height}${this.props.unit || 'px'};"${RedAgate.htmlAttributesRenderer(this.props, new Set(['id', 'src', 'unit', 'width', 'height'])).attrs}></img><script>document.getElementById("${id}").src=(Function("return this")())['#img_asset_mb3vhWjBUxDX__${this.props.use}'];</script>`;
            }
            else {
                r = `<img src="${this.content}" style="width:${this.props.width}${this.props.unit || 'px'};height:${this.props.height}${this.props.unit || 'px'};"${RedAgate.htmlAttributesRenderer(this.props, new Set(['src', 'unit', 'width', 'height'])).attrs}></img>`;
            }
        }
        return r;
    }
    renderAsSvg(canvas) {
        const img = (this.props.use !== void 0 && this.props.use !== null && this.props.use !== "") ?
            "#" + this.props.use :
            canvas.registerImage({ url: this.content, x: 0, y: 0, width: 1, height: 1 }, this.props.id);
        if (!this.props.asAsset) {
            canvas.drawImage(img, this.props.x || 0, this.props.y || 0, this.props.width, this.props.height);
        }
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (this.props.srcContext !== void 0 && this.props.srcContext !== null && this.props.srcContext !== "") {
            this.content = this.getContext(contexts, Shape_1.CONTEXT_ASSET_ + this.props.srcContext);
        }
        if (canvas) {
            this.renderAsSvg(canvas);
            return '';
        }
        else {
            return this.renderAsHtml(contexts);
        }
    }
}
exports.Image = Image;
class Script extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.content = '';
    }
    defer() {
        if (this.props.src === void 0 || this.props.src === null || this.props.src === '') {
            return Promise.resolve();
        }
        else if (this.props.noDownload) {
            return Promise.resolve();
        }
        else {
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher_1.FileFetcher.fetchLocation(this.props.src)
                        .then((result) => {
                        this.content = TextEncoding_1.TextEncoding.decodeUtf8(result.data);
                        resolve();
                    })
                        .catch((e) => {
                        Logger_1.Logger.log("Script#defer:catch:" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger_1.Logger.log("Script#defer:catch:" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    render(contexts, children) {
        let r = '';
        if (this.props.noDownload) {
            r = `<script${RedAgate.htmlAttributesRenderer(this.props, new Set([])).attrs}></script>`;
        }
        else {
            r = `<script${RedAgate.htmlAttributesRenderer(this.props, new Set(['src'])).attrs}>${this.content}</script>`;
        }
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (canvas) {
            canvas.registerCustomAsset(r);
            return '';
        }
        else {
            return r;
        }
    }
}
exports.Script = Script;
class Style extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.content = '';
    }
    defer() {
        if (this.props.src === void 0 || this.props.src === null || this.props.src === '') {
            return Promise.resolve();
        }
        else if (this.props.noDownload) {
            return Promise.resolve();
        }
        else {
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher_1.FileFetcher.fetchLocation(this.props.src)
                        .then((result) => {
                        this.content = TextEncoding_1.TextEncoding.decodeUtf8(result.data);
                        const reSplit = /(url\(.+?\))/;
                        const reReplace = /^url\((.+?)\)$/;
                        const fragments = this.content.split(reSplit);
                        const promises = [];
                        const indexes = [];
                        for (let i = 0; i < fragments.length; i++) {
                            const m = reReplace.exec(fragments[i]);
                            if (m && !(m[1]).startsWith('data:')) {
                                promises.push(FileFetcher_1.FileFetcher.fetchLocation(Uri_1.Uri.join(this.props.src, m[1]))
                                    .then((fmntResult) => {
                                    // Warning: Don't insert newlines to base64 encoded font!
                                    // Fonts will not rendered if you insert it.
                                    fragments[i] = fragments[i].replace(reReplace, `url(${`data:${fmntResult.contentType};charset=utf-8;base64,` + Base64_1.Base64.encode(fmntResult.data)})`);
                                }));
                                indexes.push(i);
                            }
                        }
                        Promise.all(promises)
                            .then(() => {
                            this.content = fragments.join('');
                            resolve();
                        })
                            .catch((e) => {
                            Logger_1.Logger.log("Style#defer:catch(1):" + e);
                            reject(e);
                        });
                    })
                        .catch((e) => {
                        Logger_1.Logger.log("Style#defer:catch(2):" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger_1.Logger.log("Style#defer:catch(3):" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    render(contexts, children) {
        let r = '';
        if (this.props.noDownload) {
            r = `<style type="text/css">@import url("${this.props.src}");</style>`;
        }
        else {
            r = `<style${RedAgate.htmlAttributesRenderer(this.props, new Set(['src'])).attrs}>${this.content}</style>`;
        }
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (canvas) {
            canvas.registerCustomAsset(r);
            return '';
        }
        else {
            return r;
        }
    }
}
exports.Style = Style;
exports.Font = Style;
class SingleFont extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.content = '';
    }
    defer() {
        if (this.props.src === void 0 || this.props.src === null || this.props.src === '') {
            return Promise.resolve();
        }
        else if (this.props.noDownload) {
            return Promise.resolve();
        }
        else {
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher_1.FileFetcher.fetchLocation(this.props.src)
                        .then((result) => {
                        this.content = `data:${result.contentType};charset=utf-8;base64,` + Base64_1.Base64.encode(result.data);
                        resolve();
                    })
                        .catch((e) => {
                        Logger_1.Logger.log("SingleFont#defer:catch:" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger_1.Logger.log("SingleFont#defer:catch:" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    render(contexts, children) {
        let r = '';
        if (this.props.noDownload) {
        }
        else {
        }
        r = `<style type="text/css">@font-face{font-family:${this.props.fontFamily};font-style:${this.props.fontStyle};font-weight:${this.props.fontWeight};src:${this.props.localNames ? this.props.localNames.map(x => ` local(${x})`) : ''} url(${this.props.noDownload ? this.props.src : this.content});unicode-range:${this.props.unicodeRange};}</style>`;
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (canvas) {
            canvas.registerCustomAsset(r);
            return '';
        }
        else {
            return r;
        }
    }
}
exports.SingleFont = SingleFont;
//# sourceMappingURL=bundler.js.map