"use strict";
// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
Object.defineProperty(exports, "__esModule", { value: true });
exports.PrinterMarks = exports.printerMarksDefault = void 0;
const Shape_1 = require("./tags/Shape");
exports.printerMarksDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    fill: false,
    stroke: true,
    strokeColor: "black",
    lineWidth: 0.105834,
    width: 210,
    height: 297,
    cropMarkLength: 9,
    bleedMarks: true,
    northBleed: 3,
    southBleed: 3,
    eastBleed: 3,
    westBleed: 3,
    centerMarks: true,
    centerMarkArmLength: 15,
    centerMarkArmMargin: 3,
});
class PrinterMarks extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.printerMarksDefault, props));
    }
    render(contexts, children) {
        const c = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        const p = this.props;
        // Crop Marks
        {
            // H
            c.moveTo(-p.westBleed, 0);
            c.lineTo(-p.westBleed - p.cropMarkLength, 0);
            c.moveTo(-p.westBleed, p.height);
            c.lineTo(-p.westBleed - p.cropMarkLength, p.height);
            c.moveTo(p.width + p.eastBleed, 0);
            c.lineTo(p.width + p.eastBleed + p.cropMarkLength, 0);
            c.moveTo(p.width + p.eastBleed, p.height);
            c.lineTo(p.width + p.eastBleed + p.cropMarkLength, p.height);
            // V
            c.moveTo(0, -p.northBleed);
            c.lineTo(0, -p.northBleed - p.cropMarkLength);
            c.moveTo(p.width, -p.northBleed);
            c.lineTo(p.width, -p.northBleed - p.cropMarkLength);
            c.moveTo(0, p.height + p.southBleed);
            c.lineTo(0, p.height + p.southBleed + p.cropMarkLength);
            c.moveTo(p.width, p.height + p.southBleed);
            c.lineTo(p.width, p.height + p.southBleed + p.cropMarkLength);
        }
        // Bleed Marks
        if (p.bleedMarks) {
            // H
            c.moveTo(0, -p.northBleed);
            c.lineTo(-p.westBleed - p.cropMarkLength, -p.northBleed);
            c.moveTo(0, p.height + p.southBleed);
            c.lineTo(-p.westBleed - p.cropMarkLength, p.height + p.southBleed);
            c.moveTo(p.width, -p.northBleed);
            c.lineTo(p.width + p.eastBleed + p.cropMarkLength, -p.northBleed);
            c.moveTo(p.width, p.height + p.southBleed);
            c.lineTo(p.width + p.eastBleed + p.cropMarkLength, p.height + p.southBleed);
            // V
            c.moveTo(-p.westBleed, 0);
            c.lineTo(-p.westBleed, -p.northBleed - p.cropMarkLength);
            c.moveTo(p.width + p.eastBleed, 0);
            c.lineTo(p.width + p.eastBleed, -p.northBleed - p.cropMarkLength);
            c.moveTo(-p.westBleed, p.height);
            c.lineTo(-p.westBleed, p.height + p.southBleed + p.cropMarkLength);
            c.moveTo(p.width + p.eastBleed, p.height);
            c.lineTo(p.width + p.eastBleed, p.height + p.southBleed + p.cropMarkLength);
        }
        // Center Marks
        if (p.centerMarks) {
            // H
            c.moveTo(p.width / 2, -p.northBleed);
            c.lineTo(p.width / 2, -p.northBleed - p.cropMarkLength);
            c.moveTo(p.width / 2, p.height + p.southBleed);
            c.lineTo(p.width / 2, p.height + p.southBleed + p.cropMarkLength);
            // H arms
            c.moveTo(p.width / 2 - p.centerMarkArmLength / 2, -p.northBleed - p.centerMarkArmMargin);
            c.lineTo(p.width / 2 + p.centerMarkArmLength / 2, -p.northBleed - p.centerMarkArmMargin);
            c.moveTo(p.width / 2 - p.centerMarkArmLength / 2, p.height + p.southBleed + p.centerMarkArmMargin);
            c.lineTo(p.width / 2 + p.centerMarkArmLength / 2, p.height + p.southBleed + p.centerMarkArmMargin);
            // V
            c.moveTo(-p.westBleed, p.height / 2);
            c.lineTo(-p.westBleed - p.cropMarkLength, p.height / 2);
            c.moveTo(p.width + p.eastBleed, p.height / 2);
            c.lineTo(p.width + p.eastBleed + p.cropMarkLength, p.height / 2);
            // V arms
            c.moveTo(-p.westBleed - p.centerMarkArmMargin, p.height / 2 - p.centerMarkArmLength / 2);
            c.lineTo(-p.westBleed - p.centerMarkArmMargin, p.height / 2 + p.centerMarkArmLength / 2);
            c.moveTo(p.width + p.eastBleed + p.centerMarkArmMargin, p.height / 2 - p.centerMarkArmLength / 2);
            c.lineTo(p.width + p.eastBleed + p.centerMarkArmMargin, p.height / 2 + p.centerMarkArmLength / 2);
        }
        if (p.hFold) {
            let x = 0;
            for (const dx of p.hFold) {
                x += dx;
                c.moveTo(x, -p.northBleed);
                c.lineTo(x, -p.northBleed - p.cropMarkLength);
                c.moveTo(x, p.height + p.southBleed);
                c.lineTo(x, p.height + p.southBleed + p.cropMarkLength);
            }
        }
        if (p.vFold) {
            let y = 0;
            for (const dy of p.vFold) {
                y += dy;
                c.moveTo(-p.westBleed, y);
                c.lineTo(-p.westBleed - p.cropMarkLength, y);
                c.moveTo(p.width + p.eastBleed, y);
                c.lineTo(p.width + p.eastBleed + p.cropMarkLength, y);
            }
        }
        return ``;
    }
}
exports.PrinterMarks = PrinterMarks;
//# sourceMappingURL=printing.js.map