"use strict";
// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HtmlRenderer = void 0;
const Base64_1 = require("red-agate-util/modules/convert/Base64");
const TextEncoding_1 = require("red-agate-util/modules/convert/TextEncoding");
const require_dynamic_1 = require("red-agate-util/modules/runtime/require-dynamic");
const app_1 = require("./app");
class HtmlRenderer {
    static get rendererPackageName() {
        return HtmlRenderer._rendererPackageName;
    }
    static set rendererPackageName(name) {
        HtmlRenderer._rendererPackageName = name;
    }
    static get launchOptions() {
        return HtmlRenderer._launchOptions;
    }
    static set launchOptions(opts) {
        HtmlRenderer._launchOptions = opts;
    }
    static writeToTempFile(html, tmpPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const path = require_dynamic_1.default('path');
            const fs = require_dynamic_1.default('fs');
            const util = require_dynamic_1.default('util');
            for (let i = 0; i < 3; i++) {
                let tmp = tmpPath;
                if (tmp.includes('*')) {
                    const crypto = require_dynamic_1.default('crypto');
                    tmp = tmp.replace(/\*/g, Base64_1.Base64.encode(crypto.randomBytes(6)).replace(/\+/g, 'A').replace(/\//g, 'B'));
                }
                let url = '';
                const tmpFullPath = path.resolve(tmp);
                const fd = yield util.promisify(fs.open)(tmpFullPath, 'ax');
                try {
                    yield util.promisify(fs.writeFile)(fd, html, 'utf8');
                    let p = encodeURI(tmpFullPath.replace(/\\/g, '/'));
                    if (!p.startsWith('/')) {
                        // Windows absolute paths except UNC paths.
                        p = '/' + p;
                    }
                    url = 'file://' + p;
                }
                finally {
                    yield util.promisify(fs.close)(fd);
                }
                return { tmpFullPath, url };
            }
            throw new Error("HtmlRenderer#writeToTempFile: Can't create a temp file.");
        });
    }
    static _toPdf(html, navigateOptions, pdfOptions, tmpPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const puppeteer = require_dynamic_1.default(HtmlRenderer._rendererPackageName);
            let buffer = null;
            let browser = null;
            let tmpFullPath = '';
            let url = '';
            try {
                if (tmpPath) {
                    ({ tmpFullPath, url } = yield HtmlRenderer.writeToTempFile(html, tmpPath));
                }
                else {
                    url = 'data:text/html;base64,' + Base64_1.Base64.encode(TextEncoding_1.TextEncoding.encodeToUtf8(html));
                }
                browser = yield puppeteer.launch(HtmlRenderer._launchOptions);
                const page = yield browser.newPage();
                yield page.goto(url, Object.assign({ waitUntil: 'load' }, navigateOptions || {}));
                buffer = yield page.pdf(Object.assign({
                    width: '210mm',
                    height: '297mm',
                    printBackground: true,
                }, pdfOptions || {}));
            }
            finally {
                try {
                    if (browser) {
                        yield browser.close();
                    }
                }
                catch (e) { }
                try {
                    if (tmpPath && tmpFullPath) {
                        const fs = require_dynamic_1.default('fs');
                        const util = require_dynamic_1.default('util');
                        yield util.promisify(fs.unlink)(tmpFullPath);
                    }
                }
                catch (e) { }
            }
            return buffer;
        });
    }
    static toPdf(html, navigateOptions, pdfOptions, tmpPath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof html === 'string') {
                return HtmlRenderer._toPdf(html, navigateOptions, pdfOptions, tmpPath);
            }
            else {
                return HtmlRenderer._toPdf(yield html, navigateOptions, pdfOptions, tmpPath);
            }
        });
    }
    static toPdfHandler(handler, navigateOptions, pdfOptions, tmpPath) {
        return app_1.Lambdas.pipe(handler, (html, ctx, cb) => {
            HtmlRenderer.toPdf(html, navigateOptions, pdfOptions, tmpPath)
                .then(buf => cb(null, buf))
                .catch(err => cb(err, null));
        });
    }
    static _toImage(html, navigateOptions, imageOptions, tmpPath) {
        return __awaiter(this, void 0, void 0, function* () {
            const puppeteer = require_dynamic_1.default(HtmlRenderer._rendererPackageName);
            let buffer = null;
            let browser = null;
            let tmpFullPath = '';
            let url = '';
            try {
                if (tmpPath) {
                    ({ tmpFullPath, url } = yield HtmlRenderer.writeToTempFile(html, tmpPath));
                }
                else {
                    url = 'data:text/html;base64,' + Base64_1.Base64.encode(TextEncoding_1.TextEncoding.encodeToUtf8(html));
                }
                browser = yield puppeteer.launch(HtmlRenderer._launchOptions);
                const page = yield browser.newPage();
                yield page.goto(url, Object.assign({ waitUntil: 'load' }, navigateOptions || {}));
                yield page.emulateMediaType('print');
                buffer = yield page.screenshot(Object.assign({
                    type: 'png',
                    fullPage: true,
                    omitBackground: false,
                }, imageOptions || {}));
            }
            finally {
                try {
                    if (browser) {
                        yield browser.close();
                    }
                }
                catch (e) { }
                try {
                    if (tmpPath && tmpFullPath) {
                        const fs = require_dynamic_1.default('fs');
                        const util = require_dynamic_1.default('util');
                        yield util.promisify(fs.unlink)(tmpFullPath);
                    }
                }
                catch (e) { }
            }
            return buffer;
        });
    }
    static toImage(html, navigateOptions, imageOptions, tmpPath) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof html === 'string') {
                return HtmlRenderer._toImage(html, navigateOptions, imageOptions, tmpPath);
            }
            else {
                return HtmlRenderer._toImage(yield html, navigateOptions, imageOptions, tmpPath);
            }
        });
    }
    static toImageHandler(handler, navigateOptions, imageOptions, tmpPath) {
        return app_1.Lambdas.pipe(handler, (html, ctx, cb) => {
            HtmlRenderer.toImage(html, navigateOptions, imageOptions, tmpPath)
                .then(buf => cb(null, buf))
                .catch(err => cb(err, null));
        });
    }
}
exports.HtmlRenderer = HtmlRenderer;
// tslint:disable-next-line:variable-name
HtmlRenderer._rendererPackageName = 'puppeteer';
// tslint:disable-next-line:variable-name
HtmlRenderer._launchOptions = void 0;
//# sourceMappingURL=renderer.js.map