/** @jsx RedAgate.createElement */
import * as RedAgate from './red-agate';
import { SvgCanvas, SvgTextAttributes, TextAlignValue, TextBaselineValue } from 'red-agate-svg-canvas/modules/drawing/canvas/SvgCanvas';
import { ShapeBaseProps, ShapeProps, Shape, AmbientProps, ImagingShapeBasePropsMixin } from './tags/Shape';
export interface SvgProps extends ShapeBaseProps, ImagingShapeBasePropsMixin {
    width: number;
    height: number;
    unit?: string;
    alt?: string;
    template?: string;
    templateUrl?: string;
}
export declare class Svg extends RedAgate.RedAgatePhantomComponent<SvgProps> {
    constructor(props: SvgProps);
    private template;
    defer(): Promise<void>;
    toImgTag(): string;
    toElementStyle(): string;
    toDataUrl(): string;
    toSvg(): string;
    toRendered(): string;
    beforeRender(contexts: Map<string, any>): void;
    render(contexts: Map<string, any>, children: string): string;
    afterRender(contexts: Map<string, any>): void;
}
export declare const ambientPropsDefault: AmbientProps;
export declare class Ambient extends Shape<AmbientProps> {
    constructor(props: AmbientProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface ArcProps extends ShapeProps {
    r: number;
    startDeg: number;
    endDeg: number;
    anticlockwise?: boolean;
}
export declare const arcPropsDefault: ArcProps;
export declare class Arc extends Shape<ArcProps> {
    constructor(props: ArcProps);
    render(contexts: Map<string, any>, children: string): string;
}
export declare type CanvasRenderFn = (canvas: SvgCanvas) => void;
export interface CanvasProps extends ShapeProps {
    children?: CanvasRenderFn | RedAgate.RedAgateNode | RedAgate.RedAgateNode[];
}
export declare const canvasPropsDefault: CanvasProps;
export declare class Canvas extends Shape<CanvasProps> {
    constructor(props: CanvasProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface CircleProps extends ShapeProps {
    r: number;
    anticlockwise?: boolean;
}
export declare const circlePropsDefault: CircleProps;
export declare class Circle extends Shape<CircleProps> {
    constructor(props: CircleProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface CurveProps extends ShapeProps {
    move?: boolean;
    points: number[];
}
export declare const curvePropsDefault: CurveProps;
export declare class Curve extends Shape<CurveProps> {
    constructor(props: CurveProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface GridLineProps extends ShapeProps {
    startX?: number;
    endX?: number;
    startY?: number;
    endY?: number;
    gridSize?: number;
    bleed?: number;
}
export interface GridLinePropsNoUndefined extends ShapeProps {
    startX: number;
    endX: number;
    startY: number;
    endY: number;
    gridSize: number;
    bleed: number;
}
export declare const gridLinePropsDefault: GridLinePropsNoUndefined;
export declare class GridLine extends Shape<GridLineProps> {
    constructor(props: GridLineProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface GroupProps extends ShapeProps {
    scaleX?: number;
    scaleY?: number;
    rotationDeg?: number;
    tm?: number[];
}
export declare class Group extends Shape<GroupProps> {
    constructor(props: GroupProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface LineProps extends ShapeProps {
    move?: boolean;
    points: number[];
}
export declare const linePropsDefault: LineProps;
export declare class Line extends Shape<LineProps> {
    constructor(props: LineProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface PathProps extends ShapeProps {
    close?: boolean;
    clip?: boolean;
    width?: number;
    height?: number;
}
export declare const pathPropsDefault: PathProps;
export declare class Path extends Shape<PathProps> {
    constructor(props: PathProps);
    beforeRender(contexts: Map<string, any>): void;
    render(contexts: Map<string, any>, children: string): string;
    afterRender(contexts: Map<string, any>): void;
}
export interface PieProps extends ShapeProps {
    r: number;
    startDeg: number;
    endDeg: number;
    anticlockwise?: boolean;
}
export declare const piePropsDefault: PieProps;
export declare class Pie extends Shape<PieProps> {
    constructor(props: PieProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface PolygonProps extends ShapeProps {
    points: number[];
}
export declare const polygonPropsDefault: PolygonProps;
export declare class Polygon extends Shape<PolygonProps> {
    constructor(props: PolygonProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface RectProps extends ShapeProps {
    width: number;
    height: number;
}
export declare const rectPropsDefault: RectProps;
export declare class Rect extends Shape<RectProps> {
    constructor(props: RectProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface RoundRectProps extends ShapeProps {
    width: number;
    height: number;
    r: number;
}
export declare const roundRectPropsDefault: RoundRectProps;
export declare class RoundRect extends Shape<RoundRectProps> {
    constructor(props: RoundRectProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface SvgAssetFragmentProps extends ShapeProps {
}
export declare const svgAssetFragmentPropsDefault: SvgFragmentProps;
export declare class SvgAssetFragment extends Shape<SvgAssetFragmentProps> {
    constructor(props: CanvasProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface SvgFragmentProps extends ShapeProps {
}
export declare const svgFragmentPropsDefault: SvgFragmentProps;
export declare class SvgFragment extends Shape<SvgFragmentProps> {
    constructor(props: CanvasProps);
    render(contexts: Map<string, any>, children: string): string;
}
export interface TextProps extends ShapeProps, SvgTextAttributes {
    font?: string;
    textAlign?: TextAlignValue;
    textBaseline?: TextBaselineValue;
    text: string;
}
export declare const textPropsDefault: TextProps;
export declare class Text extends Shape<TextProps> {
    constructor(props: TextProps);
    protected fillStrokeText(canvas: SvgCanvas): void;
    render(contexts: Map<string, any>, children: string): string;
    afterRender(contexts: Map<string, any>): void;
}
export interface SvgImpositionProps<T, S> extends RedAgate.ComponentProps {
    items: T[];
    scope?: S;
    paperWidth: number;
    paperHeight: number;
    cols: number;
    rows: number;
    itemRotationDeg?: number | number[];
    itemTranslation?: number[] | number[][];
    itemScale?: number[] | number[][];
}
export declare class SvgImposition<T, S> extends RedAgate.RedAgateComponent<SvgImpositionProps<T, S>> {
    constructor(props: SvgImpositionProps<T, S>);
    transform(): RedAgate.RedAgateNode[];
}
