"use strict";
// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
Object.defineProperty(exports, "__esModule", { value: true });
exports.SvgImposition = exports.Text = exports.textPropsDefault = exports.SvgFragment = exports.svgFragmentPropsDefault = exports.SvgAssetFragment = exports.svgAssetFragmentPropsDefault = exports.RoundRect = exports.roundRectPropsDefault = exports.Rect = exports.rectPropsDefault = exports.Polygon = exports.polygonPropsDefault = exports.Pie = exports.piePropsDefault = exports.Path = exports.pathPropsDefault = exports.Line = exports.linePropsDefault = exports.Group = exports.GridLine = exports.gridLinePropsDefault = exports.Curve = exports.curvePropsDefault = exports.Circle = exports.circlePropsDefault = exports.Canvas = exports.canvasPropsDefault = exports.Arc = exports.arcPropsDefault = exports.Ambient = exports.ambientPropsDefault = exports.Svg = void 0;
/** @jsx RedAgate.createElement */
const RedAgate = require("./red-agate");
const SvgCanvas_1 = require("red-agate-svg-canvas/modules/drawing/canvas/SvgCanvas");
const TextEncoding_1 = require("red-agate-util/modules/convert/TextEncoding");
const FileFetcher_1 = require("red-agate-util/modules/io/FileFetcher");
const Logger_1 = require("red-agate-util/modules/io/Logger");
const Shape_1 = require("./tags/Shape");
class Svg extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.template = null;
        if (props.template) {
            this.template = props.template;
        }
    }
    defer() {
        if (this.template !== null) {
            return Promise.resolve();
        }
        else if (this.props.templateUrl === void 0 || this.props.templateUrl === null || this.props.templateUrl === '') {
            return Promise.resolve();
        }
        else {
            const url = this.props.templateUrl;
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher_1.FileFetcher.fetchLocation(url)
                        .then((result) => {
                        this.template = TextEncoding_1.TextEncoding.decodeUtf8(result.data);
                        resolve();
                    })
                        .catch((e) => {
                        Logger_1.Logger.log("Svg#defer:catch:" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger_1.Logger.log("Svg#defer:catch:" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    toImgTag() {
        return Shape_1.toImgTag(this);
    }
    toElementStyle() {
        return Shape_1.toElementStyle(this);
    }
    toDataUrl() {
        return Shape_1.toDataUrl(this);
    }
    toSvg() {
        return Shape_1.toSvg(this);
    }
    toRendered() {
        return RedAgate.renderAsHtml_noDefer(this);
    }
    beforeRender(contexts) {
        this.setContext(contexts, Shape_1.CONTEXT_SVG_CANVAS, this.template === null ? new SvgCanvas_1.SvgCanvas() : SvgCanvas_1.SvgCanvas.fromTemplate(this.template));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        return Shape_1.renderSvgCanvas(this.props, canvas, this.props.width, this.props.height);
    }
    afterRender(contexts) {
        this.unsetContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
    }
}
exports.Svg = Svg;
exports.ambientPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {});
class Ambient extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.ambientPropsDefault, props));
        this.isAmbient = true;
        this.isGroup = false;
    }
    render(contexts, children) {
        return ``;
    }
}
exports.Ambient = Ambient;
exports.arcPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    r: 1,
    startDeg: 0,
    endDeg: 180,
    anticlockwise: true
});
class Arc extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.arcPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.arc(this.props.r, this.props.r, this.props.r, this.props.startDeg / 180 * Math.PI, this.props.endDeg / 180 * Math.PI, this.props.anticlockwise);
        return ``;
    }
}
exports.Arc = Arc;
exports.canvasPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {});
class Canvas extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        const fn = (Array.isArray(this.props.children) ?
            this.props.children.find(x => typeof x === 'function') :
            this.props.children);
        fn(canvas);
        return ``;
    }
}
exports.Canvas = Canvas;
exports.circlePropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    r: 1,
    anticlockwise: true
});
class Circle extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.circlePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.arc(this.props.r, this.props.r, this.props.r, 0, 2 * Math.PI, this.props.anticlockwise);
        canvas.closePath();
        return ``;
    }
}
exports.Circle = Circle;
exports.curvePropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    move: true,
    points: []
});
class Curve extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.curvePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (this.props.move)
            canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = this.props.move ? 2 : 0; i < this.props.points.length; i += 6) {
            canvas.bezierCurveTo(this.props.points[i + 0], this.props.points[i + 1], this.props.points[i + 2], this.props.points[i + 3], this.props.points[i + 4], this.props.points[i + 5]);
        }
        return ``;
    }
}
exports.Curve = Curve;
exports.gridLinePropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    lineWidth: 0.1,
    fill: false,
    stroke: true,
    strokeColor: "#ccc",
    startX: 0,
    endX: 210,
    startY: 0,
    endY: 296,
    gridSize: 5,
    bleed: 13
});
class GridLine extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.gridLinePropsDefault, props));
    }
    render(contexts, children) {
        const props = this.props;
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        const m1 = Math.ceil(props.bleed / props.gridSize) * props.gridSize;
        const mX = Math.ceil((props.endX - props.startX + props.bleed) / props.gridSize) * props.gridSize;
        const mY = Math.ceil((props.endY - props.startY + props.bleed) / props.gridSize) * props.gridSize;
        for (let dx = props.startX - m1; dx <= mX; dx += props.gridSize) {
            canvas.moveTo(props.startX + dx, props.startY - m1);
            canvas.lineTo(props.startX + dx, props.startY + mY);
        }
        for (let dy = props.startY - m1; dy <= mY; dy += props.gridSize) {
            canvas.moveTo(props.startX - m1, props.startY + dy);
            canvas.lineTo(props.startX + mX, props.startY + dy);
        }
        canvas.rect(props.startX || 0, props.startY || 0, props.endX - props.startX, props.endY - props.startY);
        return ``;
    }
}
exports.GridLine = GridLine;
class Group extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, {}, props));
        this.isAmbient = true;
    }
    render(contexts, children) {
        return ``;
    }
}
exports.Group = Group;
exports.linePropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    move: true,
    points: []
});
class Line extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.linePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (this.props.move)
            canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = this.props.move ? 2 : 0; i < this.props.points.length; i += 2) {
            canvas.lineTo(this.props.points[i], this.props.points[i + 1]);
        }
        return ``;
    }
}
exports.Line = Line;
exports.pathPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    close: false,
    clip: false,
    width: 10,
    height: 10,
    paths: []
});
class Path extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.pathPropsDefault, props));
    }
    beforeRender(contexts) {
        super.beforeRender(contexts);
        this.setContext(contexts, Shape_1.CONTEXT_SVG_PATH, true);
    }
    render(contexts, children) {
        return ``;
    }
    afterRender(contexts) {
        this.unsetContext(contexts, Shape_1.CONTEXT_SVG_PATH);
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (canvas) {
            if (this.props.close) {
                canvas.closePath();
            }
            if (this.props.clip) {
                canvas.clip();
            }
            else {
                this.fillStrokePaths(canvas);
                this.restoreGraphicState(canvas);
            }
        }
    }
}
exports.Path = Path;
exports.piePropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    r: 1,
    startDeg: 0,
    endDeg: 180,
    anticlockwise: true
});
class Pie extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.piePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.moveTo(this.props.r, this.props.r);
        canvas.arc(this.props.r, this.props.r, this.props.r, this.props.startDeg / 180 * Math.PI, this.props.endDeg / 180 * Math.PI, this.props.anticlockwise);
        canvas.closePath();
        return ``;
    }
}
exports.Pie = Pie;
exports.polygonPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    points: []
});
class Polygon extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.polygonPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = 2; i < this.props.points.length; i += 2) {
            canvas.lineTo(this.props.points[i], this.props.points[i + 1]);
        }
        canvas.closePath();
        return ``;
    }
}
exports.Polygon = Polygon;
exports.rectPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    width: 10,
    height: 10
});
class Rect extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.rectPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.rect(0, 0, this.props.width, this.props.height);
        return ``;
    }
}
exports.Rect = Rect;
exports.roundRectPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    width: 10,
    height: 10,
    r: 1
});
class RoundRect extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.roundRectPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.roundRect(0, 0, this.props.width, this.props.height, this.props.r);
        return ``;
    }
}
exports.RoundRect = RoundRect;
exports.svgAssetFragmentPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {});
class SvgAssetFragment extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.registerCustomAsset(children);
        return ``;
    }
}
exports.SvgAssetFragment = SvgAssetFragment;
exports.svgFragmentPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {});
class SvgFragment extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        canvas.appendCustomContent(children);
        return ``;
    }
}
exports.SvgFragment = SvgFragment;
exports.textPropsDefault = Object.assign({}, Shape_1.shapePropsDefault, {
    text: ""
});
class Text extends Shape_1.Shape {
    constructor(props) {
        super(Object.assign({}, exports.textPropsDefault, props));
    }
    fillStrokeText(canvas) {
        if (this.props.font)
            canvas.font = this.props.font;
        if (this.props.textAlign)
            canvas.textAlign = this.props.textAlign;
        if (this.props.textBaseline)
            canvas.textBaseline = this.props.textBaseline;
        if (this.props.fill && this.props.stroke)
            canvas.fillStrokeText(this.props.text, 0, 0, this.props);
        else if (this.props.fill)
            canvas.fillText(this.props.text, 0, 0, this.props);
        else if (this.props.stroke)
            canvas.strokeText(this.props.text, 0, 0, this.props);
    }
    render(contexts, children) {
        return ``;
    }
    afterRender(contexts) {
        const canvas = this.getContext(contexts, Shape_1.CONTEXT_SVG_CANVAS);
        if (!this.isPathFragment) {
            this.fillStrokeText(canvas);
            this.restoreGraphicState(canvas);
        }
    }
}
exports.Text = Text;
class SvgImposition extends RedAgate.RedAgateComponent {
    constructor(props) {
        super(props);
    }
    transform() {
        const repeater = (Array.isArray(this.props.children) ?
            this.props.children.find(x => typeof x === 'function') :
            this.props.children);
        const a = [];
        const scope = Object.assign({}, this.props.scope || {});
        for (let i = 0; i < this.props.items.length; i++) {
            a.push(RedAgate.createElement(Group, { x: ((this.props.paperWidth / this.props.cols) * (i % this.props.cols)) +
                    (Array.isArray(this.props.itemTranslation) ?
                        (Array.isArray(this.props.itemTranslation[i]) ?
                            this.props.itemTranslation[i][0] || 0 : this.props.itemTranslation[0] || 0) :
                        0), y: ((this.props.paperHeight / this.props.rows) * (Math.floor(i / this.props.cols))) +
                    (Array.isArray(this.props.itemTranslation) ?
                        (Array.isArray(this.props.itemTranslation[i]) ?
                            this.props.itemTranslation[i][1] || 0 : this.props.itemTranslation[1] || 0) :
                        0), scaleX: (Array.isArray(this.props.itemScale) ?
                    (Array.isArray(this.props.itemScale[i]) ?
                        this.props.itemScale[i][0] || 1 : this.props.itemScale[0] || 1) :
                    1), scaleY: (Array.isArray(this.props.itemScale) ?
                    (Array.isArray(this.props.itemScale[i]) ?
                        this.props.itemScale[i][1] || 1 : this.props.itemScale[1] || 1) :
                    1), rotationDeg: Array.isArray(this.props.itemRotationDeg) ?
                    this.props.itemRotationDeg[i] || 0 :
                    this.props.itemRotationDeg || 0 }, repeater(this.props.items[i], i, this.props.items, scope)));
        }
        return a;
    }
}
exports.SvgImposition = SvgImposition;
//# sourceMappingURL=svg.js.map