export interface RedAgateElement<P> {
    type: string | ComponentClass<P>;
    props: P;
    nodeId: number | null;
    children: RedAgateNode[];
    component: Component<P> | null;
}
export declare type RedAgateText = string | number;
export declare type RedAgateChild = RedAgateElement<any> | RedAgateText;
export declare type RedAgateFragment = {} | Array<RedAgateChild | any[] | boolean>;
export declare type RedAgateNode = RedAgateChild | RedAgateFragment | string | number | boolean | null | undefined;
export interface ComponentProps {
    /** 'id' attribute of the html|xml element. */
    id?: string;
    /** name of RedAgateNode. */
    ref?: string;
    /** collection of descendant named 'ref's. */
    refs?: {
        [refName: string]: RedAgateElement<any>;
    };
    /** auto numbered unique id of the html|xml element. if 'id' is not present, use this. */
    __nodeId?: number;
    /** 'class' attribute of the html|xml element. */
    styleClass?: object | string;
    /** synonym of 'styleClass'. 'class' attribute of the html|xml element. */
    className?: object | string;
    /** synonym of 'styleClass'. 'class' attribute of the html|xml element. */
    'class'?: object | string;
    /** 'style' attribute of the html|xml element. */
    style?: object | string;
    children?: RedAgateNode | RedAgateNode[];
    dangerouslySetInnerHTML?: {
        __html: string;
    };
    setInnerText?: {
        __text: string;
    };
    accesskey?: string;
    contenteditable?: 'true' | 'false';
    contextmenu?: string;
    dir?: string;
    draggable?: 'true' | 'false' | 'auto';
    dropzone?: 'copy' | 'move' | 'link';
    hidden?: true;
    lang?: string;
    spellcheck?: 'true' | 'false';
    tabindex?: string | number;
    title?: string;
    translate?: 'yes' | 'no';
}
export interface ComponentClass<P extends ComponentProps> {
    new (props?: P): Component<P>;
}
export interface Component<P extends ComponentProps> {
    earlyConstruct?(): void;
    transform?(): RedAgateNode;
    defer?(): Promise<any>;
    beforeRender?(contexts: Map<string, any>): void;
    render?(contexts: Map<string, any>, children: string): string;
    afterRender?(contexts: Map<string, any>): void;
}
export declare type ComponentFactory<P extends ComponentProps> = ComponentClass<P> | string;
export declare namespace JSX {
    interface IntrinsicElements {
        [elemName: string]: any;
    }
    interface ElementClass extends Component<any> {
    }
    interface Element extends RedAgateElement<any> {
    }
    interface ElementAttributesProperty {
        props: any;
    }
    interface ElementChildrenAttribute {
        children: {};
    }
}
export declare abstract class RedAgateComponent<P extends ComponentProps> implements Component<P> {
    props: P;
    private savedContexts;
    constructor(props?: P);
    abstract transform(): RedAgateNode;
    defer(): Promise<void>;
    protected setContext(contexts: Map<string, any>, key: string, value: any): void;
    protected unsetContext(contexts: Map<string, any>, key: string): void;
    protected getContext(contexts: Map<string, any>, key: string): any;
}
export declare abstract class RedAgatePhantomComponent<P extends ComponentProps> extends RedAgateComponent<P> {
    transform(): {};
}
export declare function createElement<P extends ComponentProps>(type: ComponentFactory<P>, props: P | null | undefined, ...children: RedAgateNode[]): RedAgateElement<P>;
export declare function cloneElement(el: RedAgateElement<any>): RedAgateElement<any>;
export declare function createElementFromComponentInstance<P extends ComponentProps>(c: RedAgateComponent<P>): RedAgateElement<P>;
export interface TransformContext {
    counter: number;
}
export declare function transform(element: RedAgateNode, transformContext?: TransformContext): RedAgateNode;
export declare function elementStyleRenderer(style: object | string): string;
export declare function htmlAttributesRenderer(props: any, omitKeys?: Set<string>, whiteListKeys?: Set<string>): {
    attrs: string;
    children: string;
};
export declare function htmlRenderer(element: RedAgateNode, contexts: Map<string, any>): string;
export declare function renderAsHtml(element: RedAgateNode): Promise<string>;
export declare function renderAsHtml_noDefer(element: RedAgateNode): string;
export declare function render(element: RedAgateNode, container: HTMLElement, callback?: (html: string | null, error: any | null) => void): void;
export declare function renderOnAwsLambda(element: RedAgateNode, callback: (error: any | null, result: any | null) => void): void;
export interface ExpressRequest {
}
export interface ExpressResponse {
    status(code: number): ExpressResponse;
    set(field: string, value?: string): ExpressResponse;
    send(content: any): ExpressResponse;
}
export declare function renderOnExpress(element: RedAgateNode, req: ExpressRequest, res: ExpressResponse, headers?: Array<[string, string | null | undefined]>): void;
export interface FragmentProps extends ComponentProps {
}
export declare class Fragment extends RedAgatePhantomComponent<FragmentProps> {
    constructor(props: FragmentProps);
}
