// Copyright (c) 2017, Shellyl_N and Authors
// license: ISC
// https://github.com/shellyln
/** @jsx RedAgate.createElement */
import * as RedAgate from './red-agate';
import { SvgCanvas } from 'red-agate-svg-canvas/modules/drawing/canvas/SvgCanvas';
import { TextEncoding } from 'red-agate-util/modules/convert/TextEncoding';
import { FileFetcher } from 'red-agate-util/modules/io/FileFetcher';
import { Logger } from 'red-agate-util/modules/io/Logger';
import { shapePropsDefault, Shape, renderSvgCanvas, toImgTag, toElementStyle, toDataUrl, toSvg, CONTEXT_SVG_CANVAS, CONTEXT_SVG_PATH } from './tags/Shape';
export class Svg extends RedAgate.RedAgatePhantomComponent {
    constructor(props) {
        super(props);
        this.template = null;
        if (props.template) {
            this.template = props.template;
        }
    }
    defer() {
        if (this.template !== null) {
            return Promise.resolve();
        }
        else if (this.props.templateUrl === void 0 || this.props.templateUrl === null || this.props.templateUrl === '') {
            return Promise.resolve();
        }
        else {
            const url = this.props.templateUrl;
            const promise = new Promise((resolve, reject) => {
                try {
                    FileFetcher.fetchLocation(url)
                        .then((result) => {
                        this.template = TextEncoding.decodeUtf8(result.data);
                        resolve();
                    })
                        .catch((e) => {
                        Logger.log("Svg#defer:catch:" + e);
                        reject(e);
                    });
                }
                catch (e) {
                    Logger.log("Svg#defer:catch:" + e);
                    reject(e);
                }
            });
            return promise;
        }
    }
    toImgTag() {
        return toImgTag(this);
    }
    toElementStyle() {
        return toElementStyle(this);
    }
    toDataUrl() {
        return toDataUrl(this);
    }
    toSvg() {
        return toSvg(this);
    }
    toRendered() {
        return RedAgate.renderAsHtml_noDefer(this);
    }
    beforeRender(contexts) {
        this.setContext(contexts, CONTEXT_SVG_CANVAS, this.template === null ? new SvgCanvas() : SvgCanvas.fromTemplate(this.template));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        return renderSvgCanvas(this.props, canvas, this.props.width, this.props.height);
    }
    afterRender(contexts) {
        this.unsetContext(contexts, CONTEXT_SVG_CANVAS);
    }
}
export const ambientPropsDefault = Object.assign({}, shapePropsDefault, {});
export class Ambient extends Shape {
    constructor(props) {
        super(Object.assign({}, ambientPropsDefault, props));
        this.isAmbient = true;
        this.isGroup = false;
    }
    render(contexts, children) {
        return ``;
    }
}
export const arcPropsDefault = Object.assign({}, shapePropsDefault, {
    r: 1,
    startDeg: 0,
    endDeg: 180,
    anticlockwise: true
});
export class Arc extends Shape {
    constructor(props) {
        super(Object.assign({}, arcPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.arc(this.props.r, this.props.r, this.props.r, this.props.startDeg / 180 * Math.PI, this.props.endDeg / 180 * Math.PI, this.props.anticlockwise);
        return ``;
    }
}
export const canvasPropsDefault = Object.assign({}, shapePropsDefault, {});
export class Canvas extends Shape {
    constructor(props) {
        super(Object.assign({}, canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        const fn = (Array.isArray(this.props.children) ?
            this.props.children.find(x => typeof x === 'function') :
            this.props.children);
        fn(canvas);
        return ``;
    }
}
export const circlePropsDefault = Object.assign({}, shapePropsDefault, {
    r: 1,
    anticlockwise: true
});
export class Circle extends Shape {
    constructor(props) {
        super(Object.assign({}, circlePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.arc(this.props.r, this.props.r, this.props.r, 0, 2 * Math.PI, this.props.anticlockwise);
        canvas.closePath();
        return ``;
    }
}
export const curvePropsDefault = Object.assign({}, shapePropsDefault, {
    move: true,
    points: []
});
export class Curve extends Shape {
    constructor(props) {
        super(Object.assign({}, curvePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        if (this.props.move)
            canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = this.props.move ? 2 : 0; i < this.props.points.length; i += 6) {
            canvas.bezierCurveTo(this.props.points[i + 0], this.props.points[i + 1], this.props.points[i + 2], this.props.points[i + 3], this.props.points[i + 4], this.props.points[i + 5]);
        }
        return ``;
    }
}
export const gridLinePropsDefault = Object.assign({}, shapePropsDefault, {
    lineWidth: 0.1,
    fill: false,
    stroke: true,
    strokeColor: "#ccc",
    startX: 0,
    endX: 210,
    startY: 0,
    endY: 296,
    gridSize: 5,
    bleed: 13
});
export class GridLine extends Shape {
    constructor(props) {
        super(Object.assign({}, gridLinePropsDefault, props));
    }
    render(contexts, children) {
        const props = this.props;
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        const m1 = Math.ceil(props.bleed / props.gridSize) * props.gridSize;
        const mX = Math.ceil((props.endX - props.startX + props.bleed) / props.gridSize) * props.gridSize;
        const mY = Math.ceil((props.endY - props.startY + props.bleed) / props.gridSize) * props.gridSize;
        for (let dx = props.startX - m1; dx <= mX; dx += props.gridSize) {
            canvas.moveTo(props.startX + dx, props.startY - m1);
            canvas.lineTo(props.startX + dx, props.startY + mY);
        }
        for (let dy = props.startY - m1; dy <= mY; dy += props.gridSize) {
            canvas.moveTo(props.startX - m1, props.startY + dy);
            canvas.lineTo(props.startX + mX, props.startY + dy);
        }
        canvas.rect(props.startX || 0, props.startY || 0, props.endX - props.startX, props.endY - props.startY);
        return ``;
    }
}
export class Group extends Shape {
    constructor(props) {
        super(Object.assign({}, {}, props));
        this.isAmbient = true;
    }
    render(contexts, children) {
        return ``;
    }
}
export const linePropsDefault = Object.assign({}, shapePropsDefault, {
    move: true,
    points: []
});
export class Line extends Shape {
    constructor(props) {
        super(Object.assign({}, linePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        if (this.props.move)
            canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = this.props.move ? 2 : 0; i < this.props.points.length; i += 2) {
            canvas.lineTo(this.props.points[i], this.props.points[i + 1]);
        }
        return ``;
    }
}
export const pathPropsDefault = Object.assign({}, shapePropsDefault, {
    close: false,
    clip: false,
    width: 10,
    height: 10,
    paths: []
});
export class Path extends Shape {
    constructor(props) {
        super(Object.assign({}, pathPropsDefault, props));
    }
    beforeRender(contexts) {
        super.beforeRender(contexts);
        this.setContext(contexts, CONTEXT_SVG_PATH, true);
    }
    render(contexts, children) {
        return ``;
    }
    afterRender(contexts) {
        this.unsetContext(contexts, CONTEXT_SVG_PATH);
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        if (canvas) {
            if (this.props.close) {
                canvas.closePath();
            }
            if (this.props.clip) {
                canvas.clip();
            }
            else {
                this.fillStrokePaths(canvas);
                this.restoreGraphicState(canvas);
            }
        }
    }
}
export const piePropsDefault = Object.assign({}, shapePropsDefault, {
    r: 1,
    startDeg: 0,
    endDeg: 180,
    anticlockwise: true
});
export class Pie extends Shape {
    constructor(props) {
        super(Object.assign({}, piePropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.moveTo(this.props.r, this.props.r);
        canvas.arc(this.props.r, this.props.r, this.props.r, this.props.startDeg / 180 * Math.PI, this.props.endDeg / 180 * Math.PI, this.props.anticlockwise);
        canvas.closePath();
        return ``;
    }
}
export const polygonPropsDefault = Object.assign({}, shapePropsDefault, {
    points: []
});
export class Polygon extends Shape {
    constructor(props) {
        super(Object.assign({}, polygonPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.moveTo(this.props.points[0], this.props.points[1]);
        for (let i = 2; i < this.props.points.length; i += 2) {
            canvas.lineTo(this.props.points[i], this.props.points[i + 1]);
        }
        canvas.closePath();
        return ``;
    }
}
export const rectPropsDefault = Object.assign({}, shapePropsDefault, {
    width: 10,
    height: 10
});
export class Rect extends Shape {
    constructor(props) {
        super(Object.assign({}, rectPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.rect(0, 0, this.props.width, this.props.height);
        return ``;
    }
}
export const roundRectPropsDefault = Object.assign({}, shapePropsDefault, {
    width: 10,
    height: 10,
    r: 1
});
export class RoundRect extends Shape {
    constructor(props) {
        super(Object.assign({}, roundRectPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.roundRect(0, 0, this.props.width, this.props.height, this.props.r);
        return ``;
    }
}
export const svgAssetFragmentPropsDefault = Object.assign({}, shapePropsDefault, {});
export class SvgAssetFragment extends Shape {
    constructor(props) {
        super(Object.assign({}, canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.registerCustomAsset(children);
        return ``;
    }
}
export const svgFragmentPropsDefault = Object.assign({}, shapePropsDefault, {});
export class SvgFragment extends Shape {
    constructor(props) {
        super(Object.assign({}, canvasPropsDefault, props));
    }
    render(contexts, children) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        canvas.appendCustomContent(children);
        return ``;
    }
}
export const textPropsDefault = Object.assign({}, shapePropsDefault, {
    text: ""
});
export class Text extends Shape {
    constructor(props) {
        super(Object.assign({}, textPropsDefault, props));
    }
    fillStrokeText(canvas) {
        if (this.props.font)
            canvas.font = this.props.font;
        if (this.props.textAlign)
            canvas.textAlign = this.props.textAlign;
        if (this.props.textBaseline)
            canvas.textBaseline = this.props.textBaseline;
        if (this.props.fill && this.props.stroke)
            canvas.fillStrokeText(this.props.text, 0, 0, this.props);
        else if (this.props.fill)
            canvas.fillText(this.props.text, 0, 0, this.props);
        else if (this.props.stroke)
            canvas.strokeText(this.props.text, 0, 0, this.props);
    }
    render(contexts, children) {
        return ``;
    }
    afterRender(contexts) {
        const canvas = this.getContext(contexts, CONTEXT_SVG_CANVAS);
        if (!this.isPathFragment) {
            this.fillStrokeText(canvas);
            this.restoreGraphicState(canvas);
        }
    }
}
export class SvgImposition extends RedAgate.RedAgateComponent {
    constructor(props) {
        super(props);
    }
    transform() {
        const repeater = (Array.isArray(this.props.children) ?
            this.props.children.find(x => typeof x === 'function') :
            this.props.children);
        const a = [];
        const scope = Object.assign({}, this.props.scope || {});
        for (let i = 0; i < this.props.items.length; i++) {
            a.push(RedAgate.createElement(Group, { x: ((this.props.paperWidth / this.props.cols) * (i % this.props.cols)) +
                    (Array.isArray(this.props.itemTranslation) ?
                        (Array.isArray(this.props.itemTranslation[i]) ?
                            this.props.itemTranslation[i][0] || 0 : this.props.itemTranslation[0] || 0) :
                        0), y: ((this.props.paperHeight / this.props.rows) * (Math.floor(i / this.props.cols))) +
                    (Array.isArray(this.props.itemTranslation) ?
                        (Array.isArray(this.props.itemTranslation[i]) ?
                            this.props.itemTranslation[i][1] || 0 : this.props.itemTranslation[1] || 0) :
                        0), scaleX: (Array.isArray(this.props.itemScale) ?
                    (Array.isArray(this.props.itemScale[i]) ?
                        this.props.itemScale[i][0] || 1 : this.props.itemScale[0] || 1) :
                    1), scaleY: (Array.isArray(this.props.itemScale) ?
                    (Array.isArray(this.props.itemScale[i]) ?
                        this.props.itemScale[i][1] || 1 : this.props.itemScale[1] || 1) :
                    1), rotationDeg: Array.isArray(this.props.itemRotationDeg) ?
                    this.props.itemRotationDeg[i] || 0 :
                    this.props.itemRotationDeg || 0 }, repeater(this.props.items[i], i, this.props.items, scope)));
        }
        return a;
    }
}
//# sourceMappingURL=svg.js.map